@@ -3,7 +3,7 @@ from .random_replay_buffer import RandomReplayBuffer
 from .n_steps_random_replay_buffer import NStepsRandomReplayBuffer
 from .priorized_replay_buffer import PrioritizedReplayBuffer
 from .n_steps_priorized_replay_buffer import NStepsPrioritizedReplayBuffer
-from .base.params import ReplayBufferParams, RandomReplayBufferParams, ReplayBufferEntry, MemoryReplayBufferEntry, \
+from .base.params import ReplayBufferParams, RandomReplayBufferParams, \
     NStepReplayBufferParams, NStepPrioritizedReplayBufferParams, PrioritizedReplayBufferParams,\
     NStepRandomReplayBufferParams
 
