@@ -1,23 +1,4 @@
 from dataclasses import dataclass
-import typing as T
-import numpy as np
-import torch
-
-
-@dataclass
-class ReplayBufferEntry:
-    s: np.ndarray
-    s_: np.ndarray
-    a: int
-    r: float
-    final: bool
-    index: T.Union[None, int] = None
-    weight: int = 1
-
-
-@dataclass
-class MemoryReplayBufferEntry(ReplayBufferEntry):
-    m: torch.Tensor = None
 
 
 @dataclass
