@@ -1,6 +1,8 @@
-from abc import ABC
 import typing as T
-from .params import ReplayBufferEntry, ReplayBufferParams
+from abc import ABC
+
+from ...base.models import ReplayBufferEntry
+from .params import ReplayBufferParams
 from ...base.base_agent import BaseAgent
 
 
